:- module(nn_queens).

:- export
        nn_queens/2,
        none_problem/2,
        problem/2,
        problem_symmetries/2,
        gap_sbds_problem/2,
        gap_sbdd_problem/2,
        isbds_problem/2,
        static_problem/2,
        sbdd_ssb_problem/4.

:- lib(ic).
:- lib(ic_sbds).
:- lib(ic_gap_sbds).
:- lib(ic_gap_sbdd).

:- lib(ldsb).
:- lib(ldsb_syms).
:- lib(ldsb_sbds).
%:- lib(myutil).
:- lib(ldsb_static).
:- lib(ldsb_misc).

nn_queens(N, Board) :-
        dim(Board, [N,N]),
        Board #:: 1..N,
        
        ( for(I, 1, N),
          param(Board, N)
        do
          collection_to_list(Board[I, 1..N], L), alldifferent(L),
          collection_to_list(Board[1..N, I], L2), alldifferent(L2)
        ),

        ( for(D, 1, (N-2)),
          param(Board, N)
        do
          ( for(I, 1, (N-D)),
            foreach(E1, L1),
            foreach(E2, L2),
            param(Board, D)
          do
            E1 is Board[I, D+I],
            E2 is Board[D+I, I]
          ),
          alldifferent(L1),
          alldifferent(L2),

          ( for(I, 1, D+1),
            foreach(E3, L3),
            foreach(E4, L4),
            param(Board, D, N)
          do
            E3 is Board[I, D-I+2],
            E4 is Board[N-I+1, N-D+I-1]
          ),
          alldifferent(L3),
          alldifferent(L4)
        ),
          
        ( for(I, 1, N),
          foreach(E1, L1),
          foreach(E2, L2),
          param(Board,N)
        do
           E1 is Board[I,I],
           E2 is Board[I,N-I+1]
        ),
        alldifferent(L1),
        alldifferent(L2).

       
problem_symmetries(Matrix, Syms) :-
        values_interchange(Matrix, ValSym),
        diagonal_reflect(Matrix, DimSym),
        rows_reflect(Matrix, ReflectSym),
        columns_reflect(Matrix, ReflectSym2),
        Syms = [ReflectSym, ReflectSym2, ValSym, DimSym].

problem(N, Matrix) :-
        nn_queens(N, Matrix),
        problem_symmetries(Matrix, Symmetries),
        ldsb_initialise(Matrix, Symmetries).

none_problem(N, Matrix) :- nn_queens(N, Matrix).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gap_sbds_problem(N,B) :-
        nn_queens(N, B),
        ic_gap_sbds:sbds_initialise(B, [row,col], value:1..N,
                        [symmetry(d_4, [row,col], []),
                         symmetry(s_n, [value], [])],
                        []).

gap_sbdd_problem(N,B) :-
        nn_queens(N, B),
        ic_gap_sbdd:sbdd_initialise(B, [row,col], value:1..N,
                        [symmetry(d_4, [row,col], []),
                         symmetry(s_n, [value], [])],
                        []).

isbds_problem(N, M) :-
        nn_queens(N,M),
        problem_symmetries(M, Syms),
        convert_symmetries(pairs, M, Syms, SBDSSyms),
        ic_sbds:sbds_initialise(M, 2, SBDSSyms, #=, []).

static_problem(N, Matrix) :-
        nn_queens(N, Matrix),
        problem_symmetries(Matrix, Symmetries),
        post_symmetries(Matrix, Symmetries).

sbdd_ssb_problem(N, Vector, VarPart, ValPart) :-
        nn_queens(N, Matrix),
        problem_symmetries(Matrix, Symmetries),
        symmetries_to_ssb(Matrix, Symmetries, Vector, VarPart, ValPart).
